/***************************************************************************
 *   Copyright (C) 2007-2012 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include <QMessageBox>
#include "runEmbossApp.h"
#include "programWindowClass.h"

krunEmbossApp::krunEmbossApp(QWidget *parent, kembossSettings *fksettings) : QObject(parent)
{
   parentWindow = parent;
   ksettings = fksettings;
   programProgress = NULL;

   localEmbossInstallation = true;

   // Standard output as well as standard error data are both shown in the programProgress window
   programProcess.setProcessChannelMode(QProcess::MergedChannels);

   // The program progress window is updated whenever new data is available
   connect(&programProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(updateProgramProgressForm()));
   connect(&programProcess, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(processFinished(int)));
}

bool krunEmbossApp::runApp() {
   beforeRunningProgram();
   // On local EMBOSS installations, check if EMBOSS program binary exists
   if (localEmbossInstallation && !checkEmbossApp()) return false;

   // Create program progress window
   if (programProgress == NULL) {
      programProgress = new programProgressForm(ksettings);
      connect(programProgress->CancelButton, SIGNAL(clicked()), this, SLOT(killProcess()));
   }
   programProgress->ProgramOutputTextEdit->document()->clear();
   programProgress->move(parentWindow->x()+100,parentWindow->y()+150);
   programProgress->show();

   // Display command line options for EMBOSS application in program progress window
   programProgress->ProgramOutputTextEdit->append("Running "+embossApp());
   programProgress->ProgramOutputTextEdit->append(programConfiguration.join("\n"));

   runTime.start(); // Timer to measure program execution time
   if (localEmbossInstallation) runEmbossAppLocal();
   #ifdef KEMBOSS_CLIENT
      else startKembossClient();
   #endif
   return true;
}

void krunEmbossApp::runEmbossAppLocal() {
    programProcess.start(ksettings->embossBin()+embossApp()
    #ifdef Q_WS_WIN
    +".exe"
    #endif
    , programConfiguration);
}

#ifdef KEMBOSS_CLIENT

bool krunEmbossApp::startKembossClient() {
   Hostname = "192.168.178.20";
   Port = 1444;

   QString PortStr;
   PortStr.setNum(Port);
   programProgress->ProgramOutputTextEdit->append("Trying to connect to "+Hostname+" on port "+PortStr+"\n");

   tcpSocket = new QTcpSocket(this);
   connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(readEmbossOutput()));
   connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),this, SLOT(displayError(QAbstractSocket::SocketError)));

   QNetworkConfigurationManager manager;
   if (manager.capabilities() & QNetworkConfigurationManager::NetworkSessionRequired) {
       // Get saved network configuration
       QSettings settings(QSettings::UserScope, "kemboss.org", "NetworkConfiguration");
       settings.beginGroup(QLatin1String("QtNetwork"));
       const QString id = settings.value(QLatin1String("DefaultNetworkConfiguration")).toString();
       settings.endGroup();

       // If the saved network configuration is not currently discovered use the system default
       QNetworkConfiguration config = manager.configurationFromIdentifier(id);
       if ((config.state() & QNetworkConfiguration::Discovered) !=
           QNetworkConfiguration::Discovered) {
           config = manager.defaultConfiguration();
       }

       networkSession = new QNetworkSession(config, this);
       connect(networkSession, SIGNAL(opened()), this, SLOT(sessionOpened()));
       networkSession->open();
   } else return false;
   return true;
}

void krunEmbossApp::sessionOpened() {
    // Save the used configuration
    QNetworkConfiguration config = networkSession->configuration();
    QString id;
    if (config.type() == QNetworkConfiguration::UserChoice)
        id = networkSession->sessionProperty(QLatin1String("UserChoiceConfiguration")).toString();
    else
        id = config.identifier();
    QSettings settings(QSettings::UserScope, "kemboss.org", "NetworkConfiguration");
    settings.beginGroup(QLatin1String("QtNetwork"));
    settings.setValue(QLatin1String("DefaultNetworkConfiguration"), id);
    settings.endGroup();

    programProgress->ProgramOutputTextEdit->append("Connection to server established\n");

    runEmbossAppOnServer();
}

void krunEmbossApp::runEmbossAppOnServer() {
    programProgress->ProgramOutputTextEdit->append("Executing EMBOSS application on server\n");
    blockSize = 0;
    tcpSocket->abort(); // Aborting previous connections if existing
    tcpSocket->connectToHost(Hostname,Port);
}

void krunEmbossApp::readEmbossOutput() {
    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_0);

    if (blockSize == 0) {
        if (tcpSocket->bytesAvailable() < (int)sizeof(quint16))
            return;
        in >> blockSize;
    }

    if (tcpSocket->bytesAvailable() < blockSize)
        return;

    in >> currentFortune;

    programProgress->ProgramOutputTextEdit->append(currentFortune);
}

void krunEmbossApp::displayError(QAbstractSocket::SocketError socketError) {
    switch (socketError) {
    case QAbstractSocket::RemoteHostClosedError:
        break;
    case QAbstractSocket::HostNotFoundError:
        QMessageBox::information(parentWindow, tr("kemboss Client"),
                                 tr("The host was not found. Please check the "
                                    "host name and port settings."));
        break;
    case QAbstractSocket::ConnectionRefusedError:
        QMessageBox::information(parentWindow, tr("kemboss Client"),
                                 tr("The connection was refused by the peer. "
                                    "Make sure the kemboss server is running, "
                                    "and check that the host name and port "
                                    "settings are correct."));
        break;
    default:
        QMessageBox::information(parentWindow, tr("kemboss Client"),
                                 tr("The following error occurred: %1.")
                                 .arg(tcpSocket->errorString()));
    }
}

void krunEmbossApp::closeKembossClient() {
   programProgress->ProgramOutputTextEdit->append(tr("Closing connection to server."));

}

#endif

bool krunEmbossApp::checkEmbossApp() // On local EMBOSS installations, check if EMBOSS program binary exists
{
   if (embossApp() == "chaos") // To avoid conflicting binary names, on some systems chaos is renamed into em_chaos
   {
      if (QFile::exists(ksettings->embossBin()+"em_chaos"))
         {
            setEmbossApp("em_chaos");
         }
   }

   // Check wether EMBOSS application exists
   if (!QFile::exists(ksettings->embossBin()+embossApp()
   #ifdef Q_WS_WIN
   +".exe"
   #endif
   )) {
      QMessageBox::warning(parentWindow, objectName(),
                   tr("EMBOSS application ")+embossApp()+tr(" couldn't be found!\n"
                   "Please check Your kemboss settings."));
      return false;
   }
   return true;
}

void krunEmbossApp::beforeRunningProgram()
{

}

void krunEmbossApp::afterRunningProgram()
{

}

void krunEmbossApp::updateProgramProgressForm()
{
   QByteArray programOutput = programProcess.readAll();
   programProgress->ProgramOutputTextEdit->append(QString(programOutput));
   QString timeElapsed;
   timeElapsed.setNum(runTime.elapsed());
   programProgress->RuntimeLabel->setText("Runtime: "+timeElapsed+" ms");
}

void krunEmbossApp::processFinished(int exitCode)
{
   programProgress->ProgramOutputTextEdit->append(tr("Finished"));
   QString timeElapsed;
   timeElapsed.setNum(runTime.elapsed());
   programProgress->RuntimeLabel->setText("Runtime: "+timeElapsed+" ms");
   programProgress->CancelButton->setText(tr("Close"));
   if (exitCode==0) {
      if (ksettings->autoCloseProgressWindow())
         programProgress->close();
      afterRunningProgram();
      if (qobject_cast<kprogramWindow *>(parentWindow) != NULL) qobject_cast<kprogramWindow *>(parentWindow)->displayOutputFile();
   }
}

void krunEmbossApp::killProcess()
{
   if (programProcess.state() != QProcess::NotRunning) programProcess.kill();
   programProgress->close();
}
